<?php

class CDBMultipleCommandBuilder extends CDbCommandBuilder {

    public function __construct($schema) {
        parent::__construct($schema);
    }

    /**
     * Creates a multiple UPDATE command.
     */
    public function createMultipleUpdateCommand($table, array $data, array $keys) {
        return $this->composeMultipleUpdateCommand($table, $data, $keys);
    }

    protected function composeMultipleUpdateCommand($table, array $data, array $keys) {

        $this->ensureTable($table);
        $tableName = $this->getDbConnection()->quoteTableName($table->name);
        $sql = 'UPDATE ' . $tableName . ' SET ';

        $rev = $this->reversArrayNormal($data);
        $revKeys = array_keys($rev);

        $params = array();
        //for all fields
        for ($i = 0; $i < count($revKeys); $i++) {
            $sql = $sql . $this->getDbConnection()->quoteColumnName($revKeys[$i]) . ' = CASE ';
            $sql = $sql . $this->getDbConnection()->quoteColumnName($keys[0]) . ' ';
            for ($j = 0; $j < count($keys[1]); $j++) {
                $wij = (':w' . $i . '_' . $j);
                $vij = (':v' . $i . '_' . $j);
                $sql = $sql . 'WHEN ' . $wij . ' THEN ' . $vij . ' ';

                $column = $table->getColumn($keys[0]);
                $params[$wij] = $column->typecast($keys[1][$j]);
                $column = $table->getColumn($revKeys[$i]);
                $params[$vij] = $column->typecast($rev[$revKeys[$i]][$j]);
            }
            $sql = $sql . 'ELSE ' . $this->getDbConnection()->quoteColumnName($revKeys[$i]) . ' END,';
        }
        $sql = substr($sql, 0, -1);
        var_dump ($sql);

        $command = $this->getDbConnection()->createCommand($sql);

        foreach ($params as $name => $value) {
            $command->bindValue($name, $value);
        }

        return $command;
    }

    /**
     * Creates a multiple DELETE command.
     */
    public function createMultipleDeleteCommand($table, $whereColumn, $values = array()) {

        $this->ensureTable($table);
        $tableName = $this->getDbConnection()->quoteTableName($table->name);

        $params = array();
        $in = '';
        $column = $table->getColumn($whereColumn);
        //var_dump ($table); die();

        $it = 0;
        foreach ($values as $val) {
            $wij = (':w_' . $it);
            $in = $in . $wij . ',';
            $params[$wij] = $column->typecast($val);
            $it++;
        }

        $in = substr($in, 0, -1);

        $sql = "DELETE FROM $tableName WHERE $whereColumn in ($in)";
        $command = $this->getDbConnection()->createCommand($sql);

        foreach ($params as $name => $value) {
            $command->bindValue($name, $value);
        }

        return $command;
    }

    protected function reversArrayNormal($arr) {
        $res = array();
        foreach ($arr as $item) {
            foreach ($item as $key => $val) {
                if (!isset($res[$key]))
                    $res[$key] = array();
                $res[$key][] = $val;
            }
        }
        return $res;
    }

}

?>
